namespace Patterns.Core.State
{
    public class CleanGumballMachine : IGumballMachine
    {
        private readonly int gumballs;
        private IGumballMachineState currentState;
        private IGumballMachineState noQuarterState;
        private IGumballMachineState hasQuarterState;
        private IGumballMachineState soldOutState;
        private IGumballMachineState soldState;


        public CleanGumballMachine(int gumballs)
        {
            this.gumballs = gumballs;
            noQuarterState = new NoQuarterState(this);
            soldOutState = new SoldOutState(this);
            soldState = new SoldState(this);
            hasQuarterState = new HasQuarterState(this);

            currentState = (HasGumballs ? NoQuarterState : SoldOutState);
        }

        private bool HasGumballs
        {
            get { return gumballs > 0; }
        }

        public IGumballMachineState SoldState
        {
            get { return soldState; }
        }

        public IGumballMachineState SoldOutState
        {
            get { return soldOutState; }
        }

        public IGumballMachineState HasQuarterState
        {
            get { return hasQuarterState; }
        }

        public IGumballMachineState NoQuarterState
        {
            get { return noQuarterState; }
        }

        public IGumballMachineState CurrentState
        {
            set { currentState = value; }
        }


        public void InsertQuarter()
        {
            currentState.InsertQuarter();
        }

        public void EjectQuarter()
        {
            currentState.EjectQuarter();
        }

        public void TurnCrank()
        {
            currentState.TurnCrank();
        }

        public void Dispense()
        {
            currentState.Dispense();
        }
    }
}