using System;

namespace Patterns.Core.State
{
    public class MessyGumballMachine
    {
        public enum MachineState
        {
            SOLD_OUT,
            NO_QUARTER,
            HAS_QUARTER,
            SOLD
        } ;

        private MachineState currentState;
        private int gumballCount;

        public MessyGumballMachine(int gumballCount)
        {
            this.gumballCount = gumballCount;
            if (gumballCount > 0)
            {
                currentState = MachineState.NO_QUARTER;
            }
        }

        public void InsertQuarter()
        {
            if (currentState == MachineState.HAS_QUARTER)
            {
                Console.Out.WriteLine("You already inserted a quarter");
            }
            else if (currentState == MachineState.NO_QUARTER)
            {
                currentState = MachineState.HAS_QUARTER;
                Console.Out.WriteLine("You inserted a quarter");
            }
            else if (currentState == MachineState.SOLD_OUT)
            {
                Console.Out.WriteLine("Returning your quarter, the machine is sold out.");
            }
            else if (currentState == MachineState.SOLD)
            {
                Console.Out.WriteLine("Please wait, I'm dispensing a gumball");
            }
        }

        public void EjectQuarter()
        {
            if (currentState == MachineState.HAS_QUARTER)
            {
                Console.Out.WriteLine("Quarter Returned");
                currentState = MachineState.NO_QUARTER;
            }
            else if (currentState == MachineState.NO_QUARTER)
            {
                Console.Out.WriteLine("You did not insert a quarter");
            }
            else if (currentState == MachineState.SOLD_OUT)
            {
                Console.Out.WriteLine("You did not insert a quarter");
            }
            else if (currentState == MachineState.SOLD)
            {
                Console.Out.WriteLine("You already turned the crank, I can't return your money");
            }
        }

        public void TurnCrank()
        {
            if (currentState == MachineState.SOLD)
            {
                Console.Out.WriteLine("Turning twice doesn't get you another gumball!!");
            }
            else if (currentState == MachineState.NO_QUARTER)
            {
                Console.Out.WriteLine("You turned but there is not quarter present.");
            }
            else if (currentState == MachineState.SOLD_OUT)
            {
                Console.Out.WriteLine("You turned but there are not gumballs");
            }
            else if (currentState == MachineState.HAS_QUARTER)
            {
                Console.Out.WriteLine("You turned...");
                currentState = MachineState.SOLD;
                Dispense();
            }
        }

        public void Dispense()
        {
            if (currentState == MachineState.SOLD)
            {
                Console.Out.WriteLine("A gumball comes rolling out!!");
                gumballCount -= 1;
                if (gumballCount == 0)
                {
                    Console.Out.WriteLine("The machine is now out of gumballs");
                    currentState = MachineState.SOLD_OUT;
                }
                else
                {
                    currentState = MachineState.NO_QUARTER;
                }
            }
            else if (currentState == MachineState.NO_QUARTER)
            {
                Console.Out.WriteLine("You need to put in a quarter first");
            }
            else if (currentState == MachineState.SOLD_OUT)
            {
                Console.Out.WriteLine("No gumball dispensed");
            }
            else if (currentState == MachineState.HAS_QUARTER)
            {
                Console.Out.WriteLine("No gumball dispensed");
            }
        }

        public override string ToString()
        {
            return string.Format("Messy Gumball Machine\r\nInventory: {0} gumballs\r\n{1}\r\n", gumballCount, CurrentStateDescription);
        }

        private string CurrentStateDescription
        {
            get
            {
                if (currentState == MachineState.HAS_QUARTER)
                {
                    return "Machine has a  quarter";
                }
                else if (currentState == MachineState.NO_QUARTER)
                {
                    return "Machine is waiting for quarter";
                }
                else if (currentState == MachineState.SOLD_OUT)
                {
                    return "Machine is sold out";
                }
                else if (currentState == MachineState.SOLD)
                {
                    return "Machine is dispensing a gumball";
                }
                return "Oops";
            }
        }
    }
}