using System;
using System.Collections.Generic;

namespace Patterns.Core.Strategy
{
    public class ListOfPeople
    {
        public enum SortType
        {
            AgeAscending,
            AgeDescending,
            FirstName,
            Birthday
        } ;

        private List<Person> people;

        public ListOfPeople()
        {
            people = new List<Person>();
        }

        public void Add(Person person)
        {
            people.Add(person);
        }

        public Person this[int index]
        {
            get { return people[index]; }
        }

        public void Sort(SortType sortType)
        {
            switch (sortType)
            {
                case SortType.AgeAscending:
                    people.Sort(delegate(Person x, Person y)
                                    {
                                        return (x.Age - y.Age);
                                    });
                    break;

                case SortType.AgeDescending:
                    people.Sort(delegate(Person x, Person y)
                                    {
                                        return (y.Age - x.Age);
                                    });
                    break;

                case SortType.FirstName:
                    people.Sort(delegate(Person x, Person y)
                                    {
                                        return StringComparer.OrdinalIgnoreCase.Compare(x.Name, y.Name);
                                    });
                    break;
                case SortType.Birthday:
                    people.Sort(delegate(Person x, Person y)
                                    {
                                        return x.Birthdate.CompareTo(y.Birthdate);
                                    });
                    break;
            }
        }
    }
}