using System;
using System.Collections.Generic;
using Patterns.Core.Adapter;

namespace Patterns.Core.Task
{
    public class MapperRegistry : IMapperRegistry
    {
        private IDictionary<Type, IMapper> mappers;


        public MapperRegistry()
        {
            mappers = new Dictionary<Type, IMapper>();
            mappers.Add(typeof (Person), Adapt<Person>(new PersonMapper()));
            mappers.Add(typeof (MallardDuck), Adapt<MallardDuck>(new DuckMapper()));
        }

        public IDbMapper<T> GetMapperFor<T>()
        {
            return ((NonGenericMapperAdapter<T>) mappers[typeof (T)]).StronglyTypedMapper;
        }


        public IMapper GetMapperFor(Type itemType)
        {
            return mappers[itemType];
        }

        private IMapper Adapt<T>(IDbMapper<T> realMapper)
        {
            return new NonGenericMapperAdapter<T>(realMapper);
        }

        private class NonGenericMapperAdapter<T> : IMapper
        {
            private IDbMapper<T> stronglyTypedMapper;


            public IDbMapper<T> StronglyTypedMapper
            {
                get { return stronglyTypedMapper; }
            }

            public NonGenericMapperAdapter(IDbMapper<T> realMapper)
            {
                this.stronglyTypedMapper = realMapper;
            }


            public void Insert(object item)
            {
                stronglyTypedMapper.Insert((T) item);
            }

            public void Update(object item)
            {
                stronglyTypedMapper.Update((T) item);
            }

            public void Delete(object item)
            {
                stronglyTypedMapper.Delete((T) item);
            }
        }
    }
}