using System;
using NUnit.Framework;
using Patterns.Core;
using Patterns.Core.Strategy;

namespace Patterns.Test.Strategy
{
    [TestFixture]
    public class StrategyTest
    {
        [Test]
        public void ShouldSortPeopleByAgeAscending()
        {
            Person oldest = new Person("Oldest", 27);
            Person middle = new Person("Oldest", 25);
            Person baby = new Person("Oldest", 2);

            ListOfPeople listOfPeople = new ListOfPeople();
            listOfPeople.Add(oldest);
            listOfPeople.Add(middle);
            listOfPeople.Add(baby);

            listOfPeople.Sort(ListOfPeople.SortType.AgeAscending);

            Assert.AreEqual(baby, listOfPeople[0]);
            Assert.AreEqual(middle, listOfPeople[1]);
            Assert.AreEqual(oldest, listOfPeople[2]);
        }

        [Test]
        public void ShouldSortPeopleByAgeDescending()
        {
            Person oldest = new Person("Oldest", 27);
            Person middle = new Person("Oldest", 25);
            Person baby = new Person("Oldest", 2);

            ListOfPeople listOfPeople = new ListOfPeople();
            listOfPeople.Add(middle);
            listOfPeople.Add(baby);
            listOfPeople.Add(oldest);

            listOfPeople.Sort(ListOfPeople.SortType.AgeDescending);

            Assert.AreEqual(oldest, listOfPeople[0]);
            Assert.AreEqual(middle, listOfPeople[1]);
            Assert.AreEqual(baby, listOfPeople[2]);
        }

        [Test]
        public void ShouldSortPeopleByFirstName()
        {
            Person charlie = new Person("Charlie", 27);
            Person david = new Person("David", 27);
            Person peter = new Person("Peter", 27);

            ListOfPeople people = new ListOfPeople();
            people.Add(david);
            people.Add(peter);
            people.Add(charlie);


            people.Sort(ListOfPeople.SortType.FirstName);

            Assert.AreEqual(charlie, people[0]);
            Assert.AreEqual(david, people[1]);
            Assert.AreEqual(peter, people[2]);
        }

        [Test]
        public void ShouldSortPeopleByBirthDate()
        {
            Person charlie = new Person("Charlie", 27, new DateTime(2006, 1, 1));
            Person david = new Person("David", 27, new DateTime(1950, 1, 1));
            Person peter = new Person("Peter", 27, new DateTime(1960, 1, 1));

            ListOfPeople people = new ListOfPeople();
            people.Add(peter);
            people.Add(david);
            people.Add(charlie);


            people.Sort(ListOfPeople.SortType.Birthday);

            Assert.AreEqual(david, people[0]);
            Assert.AreEqual(peter, people[1]);
            Assert.AreEqual(charlie, people[2]);
        }
    }
}